% tfsymbolic - Launches a gui that performs inverse laplace transforms and
% does state space calculations, somtimes using the symbolic toolbox.

tfsym = struct;
tfsym.cnst = struct;
tfsym.cnst.bwidth = 300;
tfsym.cnst.bheight = 20;
tfsym.cnst.bvspace = 10;
tfsym.cnst.bhspace = 25;
tfsym.cnst.MARGIN = 2;

% Create Figure
tfsym.fig = figure('Units','Normalized',...
                    'Position',[0.3 0.3 0.4 0.4],...
                    'Resize','off',...
                    'NumberTitle','off',...
                    'Color',[0.9 0.9 0.9],...
                    'Name','Transfer Function Symbolic',...
                    'Menubar','none',...
                    'DockControls','off'...
                    );
               
set(tfsym.fig,'Units','pixels');
tfsym.figpos = get(tfsym.fig,'Position');
tfsym.figpos = [tfsym.figpos(1:2) tfsym.cnst.bwidth+2*tfsym.cnst.bhspace+2*tfsym.cnst.MARGIN,...
                500];
set(tfsym.fig,'Position',tfsym.figpos);


% Lay out TF entry panels
panelpos = [tfsym.cnst.MARGIN 100-tfsym.cnst.MARGIN,...
                                    tfsym.figpos(3)-2*tfsym.cnst.MARGIN tfsym.figpos(4)-100];
% Button Position vectors (from bottom)
buttonpos(1,:) = [tfsym.cnst.bhspace tfsym.cnst.bvspace tfsym.cnst.bwidth tfsym.cnst.bheight];
for ii = 2:15
   buttonpos(ii,:) = buttonpos(1,:) + [0, (ii-1)*tfsym.cnst.bheight, 0, 0]; 
end
buttonpos(end+1,:) = [buttonpos(1,1), 0, buttonpos(1,3), panelpos(4)-2*tfsym.cnst.bvspace];

% TF Entry Panel 1 - numerator/ denominator
tfsym.TFPanel(1) = uipanel('Parent',tfsym.fig,...
                        'Units','pixels',...
                        'Position',panelpos,...
                        'Title','Enter Transfer Function:',...
                        'BorderType','beveledin',...
                        'Visible','on',...
                        'BackgroundColor',get(tfsym.fig,'Color'));
    instructions = {'Enter the Numerator and Denominator of the transfer function using a vector of polynomial coefficients, or the numerator or denominator of the transfer function in symbolic form with complex variable ''s''. Enter any symbolic variables in the box labeled ''Enter Symbolic Variables.''',...
                    'ex: For numerator (s^2 + 3*kp*s + ki^2):',...
                    '     enter ''[1, 3*kp, ki^2]'' in the Numerator box ',...
                    '     and ''kp ki'' in the symbolic variables text box.',...
                    'ex: The following are all equivalent:',...
                    '         ''(s^2 + 7*s + 12)''',...
                    '         ''[1 7 12]''',...
                    'and   ''(s+4)*(s+3)''.'};
                
                    tfsym.ndinstructions = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(end,:) + [0, buttonpos(10,2), 0, -buttonpos(10,2)],...
                                            'Style','text',...
                                            'String','',...
                                            'BackgroundColor',get(tfsym.fig,'Color'),...
                                            'HorizontalAlignment','left');
                    set(tfsym.ndinstructions,'String',textwrap(tfsym.ndinstructions,instructions));
                    tfsym.symtext = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(9,:)+ [0, 0, -tfsym.cnst.bwidth/2, 0],...
                                            'Style','text',...
                                            'String','Enter Symbolic Variables',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                    tfsym.symbox = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(9,:)+ [tfsym.cnst.bwidth/2, 0, -tfsym.cnst.bwidth/2, 0],...
                                            'Style','edit',...
                                            'String','k',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.numtext = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(8,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Numerator',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                    tfsym.numbox = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:),...
                                            'Style','edit',...
                                            'String','[1]',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.dentext = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(6,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Denominator',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                    tfsym.denbox = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(5,:),...
                                            'Style','edit',...
                                            'String','[1]',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.routhbutton = uicontrol('Parent',tfsym.TFPanel(1),...
                                             'Style','Pushbutton',...
                                             'String','Routh-Hurwitz',...
                                             'Units','pixels',...
                                             'Position',buttonpos(3,:),...
                                             'Value',1,...
                                             'Callback',...
                                            ['commandwindow; RH = routheval(parsePoly(get(tfsym.denbox,''String''),get(tfsym.symbox,''String''))),'...
                                             'disp(''Routh-Hurwitz Matrix:''), pretty(RH), figure(tfsym.fig);']);
                    tfsym.ilaplacebutton = uicontrol('Parent',tfsym.TFPanel(1),...
                                            'Style','Pushbutton',...
                                            'String','Inverse Laplace Transform',...
                                            'Units','pixels',...
                                            'Position',buttonpos(1,:),...
                                            'Value',1,...
                                            'Callback',...
                                           ['commandwindow;'...
                                            'parseTF2(get(tfsym.numbox,''String''),get(tfsym.denbox,''String''),get(tfsym.symbox,''String''));'...
                                            'figure(tfsym.fig);']);

% TF Entry Panel 2 - State Space
tfsym.TFPanel(2) = uipanel('Parent',tfsym.fig,...
                        'Units','pixels',...
                        'Position',panelpos,...
                        'Title','Enter Matrix:',...
                        'BorderType','beveledin',...
                        'Visible','off',...
                        'BackgroundColor',get(tfsym.fig,'Color'));
                    
    instructions = {'Enter the Coefficient Matrices (empty matrices will give error)',...
                    '',...
                    'E.g. For a 2x2 identity matrix type in:  [1  0; 0 1]',...
                    '[1 ; 0 ; 1]  is a 3x1 column vector &  [1 0 1]  is a 1x3 row vector'};
                                     
                    tfsym.matinstructions = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(end,:) + [0, buttonpos(14,2), 0, -buttonpos(14,2)],...
                                            'Style','text',...
                                            'String','',...
                                            'BackgroundColor',get(tfsym.fig,'Color'),...
                                            'HorizontalAlignment','left');
                    set(tfsym.matinstructions,'String',textwrap(tfsym.matinstructions,instructions));
                    tfsym.Atext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(13,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','A =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.Abox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(13,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,0],...
                                            'Style','edit',...
                                            'String','[1,0;0,1]',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.Btext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(11,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','B =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.Bbox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(11,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,-2],...
                                            'Style','edit',...
                                            'String','[1;1]',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                     tfsym.Ctext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(9,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','C =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.Cbox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(9,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,-2],...
                                            'Style','edit',...
                                            'String','[1,1]',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.Dtext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','D =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.Dbox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,-2],...
                                            'Style','edit',...
                                            'String','[1]',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                     tfsym.Utext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(5,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','u =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.Ubox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(5,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,-2],...
                                            'Style','edit',...
                                            'String','1',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.ICtext = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(3,:)+[0,0,-buttonpos(1,3)*7/8,-5],...
                                            'Style','text',...
                                            'String','ICs: x(0) =',...
                                            'HorizontalAlignment','left',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                     tfsym.ICbox = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Units','pixels',...
                                            'Position',buttonpos(3,:)+[buttonpos(1,3)*1/8,0,-buttonpos(1,3)*1/8,-2],...
                                            'Style','edit',...
                                            'String','',...
                                            'HorizontalAlignment','left',...
                                            'Callback','',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.statespacebutton = uicontrol('Parent',tfsym.TFPanel(2),...
                                            'Style','Pushbutton',...
                                            'String','State Space',...
                                            'Units','pixels',...
                                            'Position',buttonpos(1,:),...
                                            'Callback',...
                                           ['commandwindow;'...
                                            'TFss(get(tfsym.Abox,''String''),get(tfsym.Bbox,''String''),get(tfsym.Cbox,''String''),get(tfsym.Dbox,''String''),get(tfsym.Ubox,''String''),get(tfsym.ICbox,''String''));'...
                                            'figure(tfsym.fig);']);

% TF Entry Panel 3 - Inverse Laplace - ZPK
tfsym.TFPanel(3) = uipanel('Parent',tfsym.fig,...
                        'Units','pixels',...
                        'Position',panelpos,...
                        'Title','Enter Zero/Pole/Gain',...
                        'BorderType','beveledin',...
                        'Visible','off',...
                        'BackgroundColor',get(tfsym.fig,'Color'));
                    
    instructions = {'Enter the zeros in the numerator and poles in the denominator (empty matrices will give error)',...
                    '',...
                    'E.g. For (s+3)*(s+4)*(s-2): Enter ''[-3, -4, 2]''',...
                    '',...
                    'Entries must be numeric.'};
                
                    tfsym.zpkinstructions = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(end,:)+[0 buttonpos(11,2), 0, -buttonpos(11,2)],...
                                            'Style','text',...
                                            'String','',...
                                            'BackgroundColor',get(tfsym.fig,'Color'),...
                                            'HorizontalAlignment','left');
                    set(tfsym.zpkinstructions,'String',textwrap(tfsym.zpkinstructions,instructions));
                    tfsym.gaintext = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(10,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Gain',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                    tfsym.gainbox = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(9,:),...
                                            'Style','edit',...
                                            'String','1',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.zerotext = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(8,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Zeros',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));                
                    tfsym.zerobox = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:),...
                                            'Style','edit',...
                                            'String','[0]',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.poletext = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(5,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Poles',...
                                            'BackgroundColor',get(tfsym.fig,'Color'));
                    tfsym.polebox = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Units','pixels',...
                                            'Position',buttonpos(5,:),...
                                            'Style','edit',...
                                            'String','[0]',...
                                            'BackgroundColor',[1 1 1]);
                    tfsym.ilaplacebuttonZPK = uicontrol('Parent',tfsym.TFPanel(3),...
                                            'Style','Pushbutton',...
                                            'String','Inverse Laplace Transform',...
                                            'Units','pixels',...
                                            'Position',buttonpos(1,:),...
                                            'Value',1,...
                                            'Callback',...
                                           ['commandwindow;'...
                                            'G = parseZPK(strvcat(get(tfsym.zerobox,''String''),get(tfsym.polebox,''String''),get(tfsym.gainbox,''String''))),'...
                                            'tf(G), G = tf(G); disp('' Inverse Laplace Transform:''),'...
                                            'simplify(ilaplace(poly2sym(G.num{1},''s'')/poly2sym(G.den{1},''s''))), figure(tfsym.fig);']);

                    
 % Control Panel and Buttons
 
 tfsym.ControlPanel = uipanel('Parent',tfsym.fig,...
                              'Units','pixels',...
                              'Position',[panelpos(1), tfsym.cnst.MARGIN,...
                                          panelpos(3), tfsym.figpos(4)-panelpos(4)-2*tfsym.cnst.MARGIN],...
                              'Title','Control Panel',...
                              'BorderType','line',...
                              'BackgroundColor',get(tfsym.fig,'Color'),...
                              'Visible','on');
                       tfsym.fcnselect = uicontrol('Parent',tfsym.ControlPanel,...
                                                    'Style','popupmenu',...
                                                    'String',{'Transfer Function Symbolic','State Space','Inverse Laplace - ZPK'},...
                                                    'Units','pixels',...
                                                    'Position',buttonpos(3,:),...
                                                    'Value',1,...
                                                    'Callback','set(tfsym.TFPanel(:),''Visible'',''off''); set(tfsym.TFPanel(get(gcbo,''Value'')),''Visible'',''on'');');

                        tfsym.closebutton = uicontrol('Parent',tfsym.ControlPanel,...
                                                    'Style','Pushbutton',...
                                                    'String','Close',...
                                                    'Units','pixels',...
                                                    'Position',buttonpos(1,:),...
                                                    'Value',1,...
                                                    'Callback','close(gcf)');

clear instructions buttonpos panelpos